@@ -1,7 +1,7 @@
 //
 // ned, https://github.com/nevdelap/ned, main.rs
 //
-// Copyright 2016-2024 Nev Delap (nevdelap at gmail)
+// Copyright 2016-2026 Nev Delap (nevdelap at gmail)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,14 +18,6 @@
 // 02110-1301, USA.
 //
 
-extern crate ansi_term;
-extern crate getopts;
-extern crate glob;
-extern crate libc;
-extern crate regex;
-extern crate time;
-extern crate walkdir;
-
 mod colors;
 mod files;
 mod ned_error;
@@ -37,18 +29,17 @@ mod source;
 mod tests;
 
 use crate::files::Files;
-use crate::ned_error::{stderr_write_file_err, NedError, NedResult};
+use crate::ned_error::{NedError, NedResult, stderr_write_file_err};
 use crate::options_with_defaults::OptionsWithDefaults;
 use crate::opts::{make_opts, usage_brief, usage_full, usage_version};
-use crate::parameters::{get_parameters, Parameters};
+use crate::parameters::{Parameters, get_parameters};
 use crate::source::Source;
-#[cfg(target_os = "windows")]
-use ansi_term::enable_ansi_support;
-use ansi_term::Colour::{Purple, Red};
+use nu_ansi_term::Color;
 use regex::{Captures, Match, Regex};
+use std::borrow::Cow;
 use std::collections::HashMap;
 use std::fs::OpenOptions;
-use std::io::{stderr, stdin, stdout, Read, Seek, SeekFrom, Write};
+use std::io::{Read, Seek, SeekFrom, Write, stderr, stdin, stdout};
 use std::iter::Iterator;
 use std::string::String;
 use std::{env, process};
@@ -61,8 +52,8 @@ fn main() {
     let exit_code = match ned(&mut output, &env::args().skip(1).collect::<Vec<String>>()) {
         Ok(exit_code) => exit_code,
         Err(err) => {
-            let _ = stderr()
-                .write_all(&format!("{}\n{}\n\n", usage_brief(), err.to_string()).into_bytes());
+            let mut e = stderr();
+            let _ = writeln!(e, "{}\n{}\n", usage_brief(), err);
             1
         }
     };
@@ -75,35 +66,49 @@ fn ned(output: &mut dyn Write, args: &[String]) -> NedResult<i32> {
     let parameters = get_parameters(&options_with_defaults)?;
 
     if parameters.version {
-        let _ = output.write_all(&format!("\n{}\n", usage_version()).into_bytes());
-        process::exit(0);
+        let _ = writeln!(output, "\n{}", usage_version());
+        return Ok(0);
     }
 
     if parameters.help {
-        let _ = output.write_all(
-            &format!("\n{}\n", usage_full(options_with_defaults.get_opts())).into_bytes(),
-        );
-        process::exit(0);
+        let _ = writeln!(output, "\n{}", usage_full(options_with_defaults.get_opts()));
+        return Ok(0);
     }
 
     if parameters.regex.is_none() {
-        let _ = stderr().write_all(&format!("\n{}\n\n", usage_brief()).into_bytes());
-        process::exit(1);
+        let mut e = stderr();
+        let _ = write!(e, "\n{}\n\n", usage_brief());
+        return Ok(1);
     }
 
-    if parameters.colors {
-        #[cfg(target_os = "windows")]
-        match enable_ansi_support() {
-            Ok(_) => {}
-            Err(_) => {
-                let _ = stderr().write_all(
-                    &"Sadly, colors are not supported in this terminal. ansi_term colors are not supported in Git Bash or Cygwin Terminal. Colors are supported in cmd.exe, PowerShell, the OS X terminal, and all Linux terminals.\n\n"
-                        .to_string()
-                        .into_bytes(),
-                );
-                process::exit(1);
-            }
-        }
+    let found_matches = process_files(output, &parameters)?;
+    Ok(if found_matches { 0 } else { 1 })
+}
+
+#[cfg(test)]
+fn ned_with_defaults(
+    output: &mut dyn Write,
+    args: &[String],
+    defaults: Option<&str>,
+) -> NedResult<i32> {
+    let options_with_defaults =
+        OptionsWithDefaults::new_with_defaults_string(make_opts(), args, defaults)?;
+    let parameters = get_parameters(&options_with_defaults)?;
+
+    if parameters.version {
+        let _ = writeln!(output, "\n{}", usage_version());
+        return Ok(0);
+    }
+
+    if parameters.help {
+        let _ = writeln!(output, "\n{}", usage_full(options_with_defaults.get_opts()));
+        return Ok(0);
+    }
+
+    if parameters.regex.is_none() {
+        let mut e = stderr();
+        let _ = write!(e, "\n{}\n\n", usage_brief());
+        return Ok(1);
     }
 
     let found_matches = process_files(output, &parameters)?;
@@ -157,20 +162,23 @@ fn process_file(
     file_name: &Option<String>,
     source: &mut Source,
 ) -> NedResult<bool> {
-    let content: String;
-    {
+    let re = parameters
+        .regex
+        .clone()
+        .expect("Bug, already checked parameters.");
+
+    if let Some(mut replacement) = parameters.replace.clone() {
+        // Replacements operate on whole files; read complete content.
         let read: &mut dyn Read = match source {
-            Source::Stdin(ref mut read) => read,
-            Source::File(ref mut file) => file,
+            Source::Stdin(read) => read,
+            Source::File(file) => file,
             #[cfg(test)]
-            Source::Cursor(ref mut cursor) => cursor,
+            Source::Cursor(cursor) => cursor,
         };
         let mut buffer = Vec::new();
         let _ = read.read_to_end(&mut buffer)?;
-        match String::from_utf8(buffer) {
-            Ok(ref parsed) => {
-                content = parsed.to_string();
-            }
+        let mut content = match String::from_utf8(buffer) {
+            Ok(parsed) => parsed,
             Err(err) => {
                 if parameters.ignore_non_utf8 {
                     return Ok(false);
@@ -178,112 +186,211 @@ fn process_file(
                     return Err(NedError::from(err));
                 }
             }
-        }
-    }
-
-    let re = parameters
-        .regex
-        .clone()
-        .expect("Bug, already checked parameters.");
-
-    if let Some(mut replacement) = parameters.replace.clone() {
+        };
         if parameters.colors {
-            replacement = Red.bold().paint(replacement.as_str()).to_string();
+            replacement = paint_red_bold(&replacement);
         }
         if parameters.case_replacements {
             replacement = replace_case_escape_sequences_with_special_strings(&replacement);
         }
-        let (content, found_matches) = replace(parameters, &re, &content, &replacement);
-        let content = if parameters.case_replacements {
-            replace_case_with_special_strings(&content)
+        let (new_content, found_matches) = replace(parameters, &re, &content, &replacement);
+        content = if parameters.case_replacements {
+            replace_case_with_special_strings(&new_content)
         } else {
-            content
+            new_content
         };
         if parameters.stdout {
             if !parameters.quiet {
                 write_file_name_and_line_number(output, parameters, file_name, None)?;
-                output.write_all(&content.into_bytes())?;
+                output.write_all(content.as_bytes())?;
             }
         } else {
             // It's not a single match in test.
             #[allow(clippy::single_match)]
             match source {
                 // A better way???
-                Source::File(ref mut file) => {
+                Source::File(file) => {
                     if found_matches {
                         file.seek(SeekFrom::Start(0))?;
-                        let bytes = &content.into_bytes();
+                        let bytes = content.as_bytes();
                         file.write_all(bytes)?;
                         file.set_len(bytes.len() as u64)?;
                     }
                 }
                 #[cfg(test)]
-                Source::Cursor(ref mut cursor) => {
+                Source::Cursor(cursor) => {
                     cursor.seek(SeekFrom::Start(0))?;
-                    cursor.write_all(&content.into_bytes())?;
+                    cursor.write_all(content.as_bytes())?;
                 }
                 _ => {}
             }
         }
         Ok(found_matches)
     } else if parameters.file_names_only {
-        let found_matches = re.is_match(&content);
-        if found_matches ^ parameters.no_match {
-            write_file_name_and_line_number(output, parameters, file_name, None)?;
+        // Prefer streaming for line-mode; read whole file only for whole-files patterns.
+        let read: &mut dyn Read = match source {
+            Source::Stdin(read) => read,
+            Source::File(file) => file,
+            #[cfg(test)]
+            Source::Cursor(cursor) => cursor,
+        };
+        if parameters.whole_files {
+            let mut buffer = Vec::new();
+            let _ = read.read_to_end(&mut buffer)?;
+            match String::from_utf8(buffer) {
+                Ok(content) => {
+                    let found_matches = re.is_match(&content);
+                    if found_matches ^ parameters.no_match {
+                        write_file_name_and_line_number(output, parameters, file_name, None)?;
+                    }
+                    Ok(found_matches)
+                }
+                Err(err) => {
+                    if parameters.ignore_non_utf8 {
+                        Ok(false)
+                    } else {
+                        Err(NedError::from(err))
+                    }
+                }
+            }
+        } else {
+            use std::io::BufRead;
+            let mut buf = std::io::BufReader::new(read);
+            let mut line = String::new();
+            let mut found_matches = false;
+            loop {
+                line.truncate(0);
+                let n = buf.read_line(&mut line)?;
+                if n == 0 {
+                    break;
+                }
+                let text = if line.ends_with('\n') {
+                    &line[..line.len() - 1]
+                } else {
+                    &line
+                };
+                if is_match_with_number_skip_backwards(parameters, &re, text) {
+                    found_matches = true;
+                    if parameters.quiet {
+                        break;
+                    }
+                }
+            }
+            if found_matches ^ parameters.no_match {
+                write_file_name_and_line_number(output, parameters, file_name, None)?;
+            }
+            Ok(found_matches)
         }
-        Ok(found_matches)
     } else if !parameters.whole_files {
+        // Stream line-mode: build context windows on the fly using before/after counters.
+        use std::collections::{HashSet, VecDeque};
+        use std::io::BufRead;
+
+        let read: &mut dyn Read = match source {
+            Source::Stdin(read) => read,
+            Source::File(file) => file,
+            #[cfg(test)]
+            Source::Cursor(cursor) => cursor,
+        };
+        let mut buf = std::io::BufReader::new(read);
+        let mut line = String::new();
+        let mut line_number: usize = 0;
         let mut found_matches = false;
-        let context_map = make_context_map(parameters, &re, &content);
-        for (index, line) in content.lines().enumerate() {
-            let line_number = index + 1;
-            found_matches |= process_text(
-                output,
-                parameters,
-                &re,
-                file_name,
-                Some(line_number),
-                line,
-                Some(&context_map),
-            )?;
-            if parameters.quiet && found_matches {
+        let mut after_remaining: usize = 0;
+        let mut before_buf: VecDeque<(usize, String)> = VecDeque::new();
+        let mut printed: HashSet<usize> = HashSet::new();
+
+        loop {
+            line.truncate(0);
+            let n = buf.read_line(&mut line)?;
+            if n == 0 {
                 break;
             }
+            line_number += 1;
+            let text = if line.ends_with('\n') {
+                &line[..line.len() - 1]
+            } else {
+                &line
+            };
+
+            if parameters.quiet
+                && !parameters.limit_matches()
+                && parameters.group.is_none()
+                && re.is_match(text)
+            {
+                return Ok(true);
+            }
+
+            let this_line_matches = is_match_with_number_skip_backwards(parameters, &re, text);
+
+            if this_line_matches {
+                // Print before-context lines not yet printed.
+                if parameters.context_before > 0 {
+                    for (ln, ctx) in before_buf.iter() {
+                        if !printed.contains(ln) {
+                            write_line(output, parameters, file_name, Some(*ln), ctx)?;
+                            printed.insert(*ln);
+                        }
+                    }
+                }
+                after_remaining = parameters.context_after;
+                // Print the matched line via existing logic (colors, groups, matches-only etc.).
+                found_matches |= process_text(
+                    output,
+                    parameters,
+                    &re,
+                    file_name,
+                    Some(line_number),
+                    text,
+                    None,
+                )?;
+                printed.insert(line_number);
+                if parameters.quiet && found_matches {
+                    break;
+                }
+            } else if after_remaining > 0 {
+                write_line(output, parameters, file_name, Some(line_number), text)?;
+                printed.insert(line_number);
+                after_remaining -= 1;
+            } else if parameters.no_match {
+                // Show unmatched lines when --no-match is specified.
+                write_line(output, parameters, file_name, Some(line_number), text)?;
+            }
+
+            // Maintain before buffer window.
+            if parameters.context_before > 0 {
+                before_buf.push_back((line_number, text.to_string()));
+                while before_buf.len() > parameters.context_before {
+                    before_buf.pop_front();
+                }
+            }
         }
         Ok(found_matches)
     } else {
+        // Whole-file processing (match/replace operate on complete content).
+        let read: &mut dyn Read = match source {
+            Source::Stdin(read) => read,
+            Source::File(file) => file,
+            #[cfg(test)]
+            Source::Cursor(cursor) => cursor,
+        };
+        let mut buffer = Vec::new();
+        let _ = read.read_to_end(&mut buffer)?;
+        let content = match String::from_utf8(buffer) {
+            Ok(parsed) => parsed,
+            Err(err) => {
+                if parameters.ignore_non_utf8 {
+                    return Ok(false);
+                } else {
+                    return Err(NedError::from(err));
+                }
+            }
+        };
         let found_matches = process_text(output, parameters, &re, file_name, None, &content, None)?;
         Ok(found_matches)
     }
 }
 
-/// Returns a vector whose capacity equals the number of lines in the file, and whose
-/// value is a boolean that indicates whether or not that line should be shown given
-/// the -C --context, -B --before, and -A --after options specified in the parameters.
-fn make_context_map(parameters: &Parameters, re: &Regex, content: &str) -> Vec<bool> {
-    let lines = content.lines().map(str::to_string).collect::<Vec<String>>();
-    let mut match_map = Vec::<bool>::with_capacity(lines.len());
-    for line in lines {
-        match_map.push(is_match_with_number_skip_backwards(parameters, re, &line));
-    }
-    let mut context_map = match_map.clone();
-    for line in 0..context_map.len() {
-        if match_map[line] {
-            // We can't use std::cmp::max() for this test because the indices are unsigned.
-            let start = if line >= parameters.context_before {
-                line - parameters.context_before
-            } else {
-                0usize
-            };
-            let end = std::cmp::min(match_map.len(), line + parameters.context_after + 1);
-            for item in context_map.iter_mut().take(end).skip(start) {
-                *item = true;
-            }
-        }
-    }
-    context_map
-}
-
 fn is_match_with_number_skip_backwards(parameters: &Parameters, re: &Regex, text: &str) -> bool {
     let start_end_byte_indices = re.find_iter(text);
     let count = start_end_byte_indices.count();
@@ -325,10 +432,10 @@ fn process_text(
         } else {
             // TODO 4: make it respect -n, -k, -b TO TEST
             // Need to get is found_matches out of this...
-            let (text, found_matches) =
+            let (colored, found_matches) =
                 color_matches_with_number_skip_backwards(parameters, re, text);
             if found_matches {
-                write_line(output, parameters, file_name, line_number, &text)?;
+                write_line(output, parameters, file_name, line_number, colored.as_ref())?;
                 return Ok(true);
             }
         }
@@ -344,6 +451,10 @@ fn process_text(
     Ok(false)
 }
 
+fn paint_red_bold(text: &str) -> String {
+    Color::Red.bold().paint(text).to_string()
+}
+
 /// Do a replace_all() or a find_iter() taking into account which of --number, --skip, and
 /// --backwards have been specified.
 fn replace(parameters: &Parameters, re: &Regex, text: &str, replace: &str) -> (String, bool) {
@@ -362,14 +473,10 @@ fn replace(parameters: &Parameters, re: &Regex, text: &str, replace: &str) -> (S
             if parameters.include_match(index, count) {
                 found_matches = true;
                 let this_replace = re.replace(_match.as_str(), replace).into_owned();
-                new_text = format!(
-                    "{}{}{}",
-                    // find_iter guarantees that start and end
-                    // are at a Unicode code point boundary.
-                    unsafe { &new_text.get_unchecked(0.._match.start()) },
-                    this_replace,
-                    unsafe { &new_text.get_unchecked(_match.end()..new_text.len()) }
-                );
+                // find_iter guarantees that start and end are at a Unicode code point boundary.
+                let prefix = &new_text[0.._match.start()];
+                let suffix = &new_text[_match.end()..];
+                new_text = format!("{}{}{}", prefix, this_replace, suffix);
             }
         }
     };
@@ -467,8 +574,8 @@ fn write_line(
     if !parameters.quiet {
         write_file_name_and_line_number(output, parameters, file_name, line_number)?;
         if !parameters.line_numbers_only && !parameters.quiet {
-            output.write_all(&text.to_string().into_bytes())?;
-            write_newline_if_replaced_text_ends_with_newline(output, text)?;
+            output.write_all(text.as_bytes())?;
+            write_ensuring_trailing_newline(output, text)?;
         }
     }
     Ok(())
@@ -505,7 +612,7 @@ fn write_groups(
                         )?;
                         wrote_file_name = true;
                     }
-                    output.write_all(&text.to_string().into_bytes())?;
+                    output.write_all(text.as_ref().as_bytes())?;
                 } else {
                     break;
                 }
@@ -513,7 +620,7 @@ fn write_groups(
         }
     }
     if !parameters.quiet && found_matches {
-        output.write_all(&"\n".to_string().into_bytes())?;
+        output.write_all(b"\n")?;
     }
     Ok(found_matches)
 }
@@ -539,16 +646,16 @@ fn write_matches(
                 write_file_name_and_line_number(output, parameters, file_name, line_number)?;
                 file_name_written = true;
             }
-            let text = color(parameters, &text[_match.start().._match.end()]);
+            let colored = color(parameters, &text[_match.start().._match.end()]);
             if !parameters.quiet {
-                output.write_all(&text.to_string().into_bytes())?;
+                output.write_all(colored.as_ref().as_bytes())?;
             } else {
                 return Ok(found_matches);
             }
         }
     }
     if file_name_written {
-        output.write_all(&"\n".to_string().into_bytes())?;
+        output.write_all(b"\n")?;
     }
     Ok(found_matches)
 }
@@ -565,7 +672,7 @@ fn write_file_name_and_line_number(
     if !parameters.quiet {
         let mut location = "".to_string();
         if !parameters.no_file_names && !parameters.line_numbers_only {
-            if let Some(ref file_name) = file_name {
+            if let Some(file_name) = file_name {
                 location.push_str(file_name);
             }
         }
@@ -588,58 +695,55 @@ fn write_file_name_and_line_number(
                 },
             );
             if parameters.colors {
-                location = Purple.paint(location).to_string();
+                location = Color::Purple.paint(location).to_string();
             }
-            output.write_all(&location.into_bytes())?;
+            output.write_all(location.as_bytes())?;
         }
     }
     Ok(())
 }
 
-fn write_newline_if_replaced_text_ends_with_newline(
-    output: &mut dyn Write,
-    text: &str,
-) -> NedResult<()> {
+fn write_ensuring_trailing_newline(output: &mut dyn Write, text: &str) -> NedResult<()> {
     if !text.ends_with('\n') {
-        output.write_all(&"\n".to_string().into_bytes())?;
+        output.write_all(b"\n")?;
     }
     Ok(())
 }
 
 // TODO: use Cows to reduce allocations in the color*() functions.
 
-fn color_matches_with_number_skip_backwards(
+fn color_matches_with_number_skip_backwards<'a>(
     parameters: &Parameters,
     re: &Regex,
-    text: &str,
-) -> (String, bool) {
-    let (new_text, found_matches) = replace(
-        parameters,
-        re,
-        text,
-        Red.bold().paint("$0").to_string().as_str(),
-    );
+    text: &'a str,
+) -> (Cow<'a, str>, bool) {
     if parameters.colors {
-        (new_text, found_matches)
+        let (new_text, found_matches) =
+            replace(parameters, re, text, paint_red_bold("$0").as_str());
+        (Cow::Owned(new_text), found_matches)
     } else {
-        (text.to_string(), found_matches)
+        let found_matches = is_match_with_number_skip_backwards(parameters, re, text);
+        (Cow::Borrowed(text), found_matches)
     }
 }
 
-fn color_matches_all(parameters: &Parameters, re: &Regex, text: &str) -> String {
+fn color_matches_all<'a>(parameters: &Parameters, re: &Regex, text: &'a str) -> Cow<'a, str> {
     if parameters.colors {
-        re.replace_all(text, Red.bold().paint("$0").to_string().as_str())
-            .into_owned()
+        re.replace_all(text, |caps: &Captures| {
+            // Color the actual matched text via a closure to avoid building a static replacement.
+            let m = caps.get(0).expect("match exists").as_str();
+            paint_red_bold(m)
+        })
     } else {
-        text.to_string()
+        Cow::Borrowed(text)
     }
 }
 
 /// Color the whole text if --colors has been specified.
-fn color(parameters: &Parameters, text: &str) -> String {
+fn color<'a>(parameters: &Parameters, text: &'a str) -> Cow<'a, str> {
     if parameters.colors {
-        Red.bold().paint(text).to_string()
+        Cow::Owned(paint_red_bold(text))
     } else {
-        text.to_string()
+        Cow::Borrowed(text)
     }
 }
