@@ -1,7 +1,7 @@
 //
 // ned, https://github.com/nevdelap/ned, tests/general.rs
 //
-// Copyright 2016-2024 Nev Delap (nevdelap at gmail)
+// Copyright 2016-2026 Nev Delap (nevdelap at gmail)
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 
 /// Just a few general tests. The specifics are tested in the other test files.
 use crate::ned;
-use std;
 
 #[test]
 fn basic_match() {
@@ -381,6 +380,16 @@ fn basic_match_line_numbers_only() {
     test(&args, expected_exit_code, &expected_screen_output);
 }
 
+#[test]
+fn basic_match_line_numbers_only_short_flag() {
+    // Regression: ensure '-l' maps to --line-numbers-only (not --follow)
+    let args = vec!["accidentally", "test", "-l"];
+    let expected_exit_code = 0;
+    let expected_screen_output = ["1\n"];
+
+    test(&args, expected_exit_code, &expected_screen_output);
+}
+
 #[test]
 fn basic_match_no_file_names() {
     let args = vec!["accidentally", "test", "--no-filenames"];
@@ -454,6 +463,16 @@ fn basic_match_line_numbers_only_no_match() {
     test(&args, expected_exit_code, &expected_screen_output);
 }
 
+#[test]
+fn basic_match_line_numbers_only_no_match_short_flag() {
+    // Regression: short '-l' with '--no-match' prints only non-matching line numbers
+    let args = vec!["secretly", "test/dir1", "-l", "--no-match"];
+    let expected_exit_code = 0;
+    let expected_screen_output = ["1\n2\n3\n4\n5\n6\n1\n2\n3\n"];
+
+    test(&args, expected_exit_code, &expected_screen_output);
+}
+
 #[test]
 fn basic_match_no_file_names_no_match() {
     let args = vec!["secretly", "test/dir1", "--no-filenames", "--no-match"];
@@ -1458,7 +1477,7 @@ fn test(args: &[&str], expected_exit_code: i32, expected_screen_output: &[&str])
         .iter()
         .map(|arg| arg.to_string())
         .collect::<Vec<String>>();
-    assert!(!expected_screen_output[0].is_empty() || expected_exit_code != 0);
+    // Allow tests that expect no output (empty expected_screen_output) with a success exit code.
 
     let mut screen_output: Vec<u8> = vec![];
 
@@ -1466,11 +1485,21 @@ fn test(args: &[&str], expected_exit_code: i32, expected_screen_output: &[&str])
 
     let screen_output = fix_output_for_windows(&String::from_utf8(screen_output).unwrap());
 
-    for part in expected_screen_output.iter() {
-        let part = fix_output_for_windows(part);
-        if !screen_output.contains(&part) {
-            println!("{:?} not in {:?}", part, screen_output);
-            unreachable!();
+    if expected_screen_output.is_empty() {
+        assert!(
+            screen_output.is_empty(),
+            "Expected no output, got: {:?}",
+            screen_output
+        );
+    } else {
+        for part in expected_screen_output.iter() {
+            let part = fix_output_for_windows(part);
+            assert!(
+                screen_output.contains(&part),
+                "Expected output missing. Looking for: {:?}\nActual output: {:?}",
+                part,
+                screen_output
+            );
         }
     }
     assert_eq!(exit_code, expected_exit_code);
@@ -1480,6 +1509,98 @@ fn fix_output_for_windows(part: &str) -> String {
     // Is sufficient for current requirements as I make tests
     // written on Linux work on Windows. Future tests will
     // be written to work on both platforms .
-    part.replace("/", &std::path::MAIN_SEPARATOR.to_string())
-        .replace("\r", "")
+    // Normalize Windows backslashes to forward slashes for comparison,
+    // and strip CR to make outputs consistent across platforms.
+    part.replace("\\", "/").replace("\r", "")
+}
+
+#[test]
+fn env_defaults_shell_parsing() {
+    // Local helper to run ned with provided defaults string to avoid global env changes.
+    fn run_with_defaults(args: &[&str], defaults: &str) -> (i32, String) {
+        let args: Vec<String> = args.iter().map(|a| a.to_string()).collect();
+        let mut screen_output: Vec<u8> = vec![];
+        let exit_code = crate::ned_with_defaults(&mut screen_output, &args, Some(defaults))
+            .expect("ned_with_defaults should succeed");
+        let out = fix_output_for_windows(&String::from_utf8(screen_output).unwrap());
+        (exit_code, out)
+    }
+
+    // 1) Colors from NED_DEFAULTS (shell-style parsing of simple flags)
+    let (exit_code, out) = run_with_defaults(&["accidentally.*hand", "test"], "--colors=always");
+    assert_eq!(exit_code, 0);
+    assert!(out.contains(
+        "\u{1b}[35mtest/file1.txt:1:\u{1b}[0mThe \u{1b}[1;31maccidentally ghastly hand\u{1b}[0m plans AN ESCAPE from a cream puff the placid widow. A slovenly\n"
+    ));
+
+    // 2) Replacement with spaces from NED_DEFAULTS (quotes respected)
+    let (exit_code, out) = run_with_defaults(
+        &["--stdout", "accidentally", "test"],
+        r"--replace 'out standing'",
+    );
+    assert_eq!(exit_code, 0);
+    assert!(out.contains(
+        "test/file1.txt:\nThe out standing ghastly hand plans AN ESCAPE from a cream puff the placid widow. A slovenly\n"
+    ));
+}
+
+#[test]
+fn streaming_context_windows() {
+    // Verify before/after contexts are emitted correctly in line-mode streaming.
+    let args = vec!["her", "test", "--recursive", "--line-numbers-only"]; // simple match baseline
+    let expected_exit_code = 0;
+    let expected_screen_output = ["3\n4\n"]; // sanity subset
+    test(&args, expected_exit_code, &expected_screen_output);
+
+    // With context before/after, ensure lines around matches appear.
+    let args = vec!["her", "test", "-B", "1", "-A", "1"]; // print neighbor lines
+    let expected_exit_code = 0;
+    let expected_screen_output = [
+        "test/longfile.txt:17:soothed by an espadrille and a fetishist, still makes a truce with her\n",
+        "test/longfile.txt:18:from an unseemly gypsy, buy an expensive gift for her a fetishist with\n",
+        "test/longfile.txt:19:a philosopher, and takes a peek at the dark side of her dilettante.\n",
+    ];
+    test(&args, expected_exit_code, &expected_screen_output);
+}
+
+#[test]
+fn colored_context_windows() {
+    // Ensure context headers are colored and matched text is highlighted.
+    let args = vec![
+        "her",
+        "test",
+        "--include",
+        "long*.txt",
+        "-B",
+        "1",
+        "-A",
+        "1",
+        "--color=always",
+    ];
+    let expected_exit_code = 0;
+    let expected_screen_output = [
+        "\u{1b}[35mtest/longfile.txt:17:\u{1b}[0m",
+        "\u{1b}[1;31mher\u{1b}[0m",
+        "\u{1b}[35mtest/longfile.txt:18:\u{1b}[0m",
+    ];
+    test(&args, expected_exit_code, &expected_screen_output);
+}
+
+#[test]
+fn quiet_with_context_no_output() {
+    // Quiet mode should suppress output even with context flags, but return success on match.
+    let args = vec![
+        "her",
+        "test",
+        "--include",
+        "long*.txt",
+        "-B",
+        "1",
+        "-A",
+        "1",
+        "--quiet",
+    ];
+    let expected_exit_code = 0;
+    let expected_screen_output: [&str; 0] = [];
+    test(&args, expected_exit_code, &expected_screen_output);
 }
